/*
 * Copyright 2013 Andrea "andynaz" Agnesse
 *
 * This file is part of "andyLib".
 *
 * "andyLib" is free software: you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation, either version 3 of the License, or (at your option) any later
 * version.
 *
 * "andyLib" is distributed in the hope that it will be useful, but WITHOUT ANY
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
 * A PARTICULAR PURPOSE.  See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along with
 * "andyLib".  If not, see <http://www.gnu.org/licenses/>.
 *
 * author e-mail: andynaz@gmail.com
 * author home page: http://andynaz.altervista.org
 * program official page: http://andynaz.altervista.org/Progetti/andyLib.shtm
 */

package it.andynaz.log;

import it.andynaz.utils.Utils;
import java.io.*;
import java.util.*;


/**
 * Configuration of a {@link Logger}.
 *
 * A configuration can be used by various loggers.
 * 
 * <p>Wrap alla the properties for a logger in a {@link Properties} object
 * (different from the one it is constructed), such as the log level or if the
 * date has to be shown.<br/>
 * Properties from the logger can be obtained using the {@link
 * #getProperty(java.lang.String)} method.</p>
 *
 * <p>Each logger takes the properties it needs from the configuration on
 * creation time. If a property is not found, it should use a default value. For
 * the list of the values searched by each kind of logger, look at its
 * documentation.</p>
 * 
 * <p>The class has also a shared config (can be thougth as a default one) used
 * to create logger without a specified configuration. This configuration can be
 * initialized using the vatious <tt>init()</tt> methods and can be retrived
 * using the {@link #getConfig()} method.</p>
 * 
 * @author andynaz
 * @version 2013/01/14
 */
public class Config{

	private Properties props = new Properties();

	/**
	 * Shared configuration.
	 * 
	 * Should be used by the loggers created without configuration.
	 */
	private static Config conf = new Config(new Properties());

	/**
	 * Creates a new configuration.
	 * 
	 * <p>A config is constructed using a Properties object; it takes all the
	 * properties starting with "andyLog." and stores them in an internal
	 * Properties object, without the prefix (for instance, the property
	 * "andyLog.level" is stored as "level").</p>
	 *
	 * @param props set of properties
	 */
	public Config(Properties props){
		Set<Object> keys = props.keySet();
		for( Object o : keys){
			if( o instanceof String ){
				String key = (String) o;
				if( key.startsWith("andyLog.") )
					this.props.setProperty(key.substring(8), props.getProperty(key));
			}
		}
	}


	/**
	 * Get a property value.
	 *
	 * @param propName name of the property
	 * @return the value of the property, or <tt>null</tt> if the property is not
	 * found
	 */
	public String getProperty(String propName){
		return this.props.getProperty(propName);
	}

	
	/**
	 * Inizialize the shared configuration with a {@link Properties} object.
	 *
	 * <p>This method can be inviked several times during execution; only the
	 * last configuration is saved.</p>
	 *
	 * @param props properties to read
	 */
	public static void init(Properties props){
		Config.conf = new Config(props);
	}

	/**
	 * Inizialize the shared configuration with a file of properties.
	 * 
	 * If the file file is not found, the error is printed in the standard
	 * output.
	 *
	 * <p>This method can be inviked several times during execution; only the
	 * last configuration is saved.</p>
	 *
	 * @param file path to the properties file
	 */
	public static void init(String file){
		Properties props = new Properties();
		try {
			props.load(new FileReader(file));
		} catch (IOException ex) {
			Utils.printException("error in creating the Congif object: file="+file, ex);
		}
		Config.init(props);
	}

	/**
	 * Inizialize the shared configuration.
	 * 
	 * This method search for a file called "config.properties" in the curretn
	 * directory. If the file file is not found, the error is printed in the
	 * standard output.
	 *
	 * <p>This method can be inviked several times during execution; only the
	 * last configuration is saved.</p>
	 */
	public static void init(){
		Config.init("config.properties");
	}
	
	/**
	 * Returns the shared configuration.
	 * 
	 * Should be used by the loggers created without configuration.
	 *
	 * <p>By default the object returned has an empty list of properties; to
	 * initialize the configuration use one of the <tt>init()</tt> methods.</p>
	 * 
	 * @return the shared configuration
	 */
	public static Config getConfig(){
		return Config.conf;
	}

	/**
	 * List all the properties on the standard output.
	 *
	 * Used only for debug pourpose.
	 */
	public void list(){
		props.list(System.out);
	}
	
}
