/*
 * Copyright 2013 Andrea "andynaz" Agnesse
 *
 * This file is part of "andyLib".
 *
 * "andyLib" is free software: you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation, either version 3 of the License, or (at your option) any later
 * version.
 *
 * "andyLib" is distributed in the hope that it will be useful, but WITHOUT ANY
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
 * A PARTICULAR PURPOSE.  See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along with
 * "andyLib".  If not, see <http://www.gnu.org/licenses/>.
 *
 * author e-mail: andynaz@gmail.com
 * author home page: http://andynaz.altervista.org
 * program official page: http://andynaz.altervista.org/Progetti/andyLib.shtm
 */

package it.andynaz.utils;

import java.io.*;
import java.util.regex.*;


/**
 * Useful methods for an elementary management of XML files.
 * 
 * To read and write XML files, methods in the {@link it.andynaz.utils.UtilsIO}
 * class can be used.
 * 
 * @author andynaz
 * @version 2013/01/14
 */
public class UtilsXML{
	// hides the constructor
	private UtilsXML(){}
	
	/**
	 * Replace the content of a tag.
	 * 
	 * Uses the {@link #replaceTag(java.lang.String, java.lang.String,
	 * java.lang.String)} method.
	 *
	 * <p>The method does not throw any Exception; if an error occours it writes
	 * a log on the standard output.</p>
	 * 
	 * @param file an XML file
	 * @param tag tag name
	 * @param content new tag content
	 */
	public static void replaceTag(File file, String tag, String content){
		try{
			String fileContent = UtilsIO.readFromFile(file).toString();
			fileContent = replaceTag(fileContent, tag, content);
			UtilsIO.writeToFile(file, fileContent);
		} catch(IOException e){
			Utils.printException("UtilsXML::replaceTag", e);
		}
	}
	
	/**
	 * Replace the content of a tag.
	 * 
	 * It is necessary for the tag to be present in the fileContent.
	 * 
	 * <p>Note: only the first occourrence will be replaced.</p>
	 * 
	 * @param fileContent content of the XML file
	 * @param tag tag name
	 * @param content new tag content
	 * @return the new content of the XML file
	 */
	public static String replaceTag(String fileContent, String tag, String content){
		return fileContent.replaceFirst("<"+tag+">.+</"+tag+">", "<"+tag+">"+content+"</"+tag+">");
	}
	
	/**
	 * Read the content of a tag from an XML file.
	 * 
	 * Uses the {@link #readTag(java.lang.String, java.lang.String)} method.
	 * 
	 * <p>Note: only the first occourrence of the tag is read.</p>
	 * 
	 * @param file an XML file
	 * @param tag tag name
	 * @return the content of the tag, or an empty string if the file or the tag
	 * cannot be found
	 * 
	 * @throws IOException if I/O errors occour
	 */
	public static String readTag(File file, String tag) throws IOException{
		return readTag(UtilsIO.readFromFile(file).toString(), tag);
	}
	
	/**
	 * Read the content of a tag from an XML file.
	 * 
	 * <p>Note: only the first occourrence of the tag is read.</p>
	 * 
	 * @param fileContent content of an XML file
	 * @param tag tag name
	 * @return the content of the tag, or an empty string if the file or the tag
	 * cannot be found
	 * 
	 * @throws NullPointerException if the <tt>tag</tt> parameter is empty
	 */
	public static String readTag(String fileContent, String tag){
		if( Utils.isEmpty(tag) )
			throw new NullPointerException("empty tag");
		String tagContent = "";
		StringBuilder content = new StringBuilder(fileContent);
		if( content==null || content.length()==0 )
			return "";
		Pattern p = Pattern.compile("<"+tag+">.+</"+tag+">");
		Matcher m = p.matcher(content);
		if( m.find() ){
			int l = tag.length();
			// remove tags <tag> and </tag>
			tagContent = m.group().substring(l+2, m.group().length()-l-3);
		}
		return tagContent;
	}
	
}
